/*--------------------------------------------------------------------------*/
/*	CoreSampleAETE.r	-	Apple Events Terminology Extension				*/
/*																			*/
/*  This 'aete' adds one additional property to the window class.  This		*/
/*	property is the window's position, the top-left hand coordinates.		*/
/*  The position code id is 'ppos'.											*/
/*	In addition, the insertion location parameter for the Create and Move	*/
/*	events have been made optional for CoreSample.  If CoreSample receives	*/
/*	either of these events without this param, it will create or move the	*/
/*	window to be the frontmost.												*/
/*--------------------------------------------------------------------------*/
#include "AERegistry.r"
#include "AppleEvents.r"
#include "AEObjects.r"
#include "CoreSample.h"

resource 'aete' (0, "CoreSample Terminology") {
	0x01,
	0x00,
	english,
	roman,
	{	/* array Suites: 1 element  */
		/* [1] */
		"CoreSample Suite",
		"Suite pertaining to CoreSample",
		kCoreSampleID,
		1,
		1,
		{	/* array Events: 3 elements */
			/* [1] */
			"run",
			"Sent to an application when it is double-clicked",
			kCoreEventClass,
			kAEOpenApplication,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			noParams,
			"No direct parameter required",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [2] */
			"quit",
			"Quit application",
			kCoreEventClass,
			kAEQuitApplication,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			noParams,
			"No direct parameter required",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [3] */
			"close",
			"Close an object",
			kAECoreSuite,
			kAEClose,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object to close",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [4] */
			"count elements",
			"Return the number of elements of a particular class within an object",
			kAECoreSuite,
			kAECountElements,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object whose elements are to be counted",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"each",
				keyAEObjectClass,
				typeType,
				"the class of the elements to be counted",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved, reserved, reserved, reserved, reserved, reserved,
				reserved, reserved, reserved, reserved, reserved, reserved,
			},
			/* [5] */
			"create",	/***NOTE: the insertion loc paramater is optional. ***/
			"Create a new element",
			kAECoreSuite,
			kAECreateElement,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 4 elements */
				/* [1] */
				"new",
				keyAEObjectClass,
				typeType,
				"the class of the new element",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"at",
				keyAEInsertHere,
				typeInsertionLoc,
				"the location at which to insert the element",
				optional,		/** default behavior is beginning **/
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"with data",
				keyAEData,
				typeWildCard,
				"the initial data for the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"with properties",
				keyAEPropData,
				typeAERecord,
				"the initial data for the properties of the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [6] */
			"delete",
			"Delete an element from an object",
			kAECoreSuite,
			kAEDelete,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the element to delete",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [7] */
			"duplicate",
			"Duplicate object",
			kAECoreSuite,
			kAEClone,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object to duplicate",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"to",
				keyAEInsertHere,
				typeInsertionLoc,
				"the new location for the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved, reserved, reserved, reserved, reserved, reserved,
				reserved, reserved, reserved, reserved, reserved, reserved,
			},
			/* [8] */
			"exists",
			"Verify if an object exists",
			kAECoreSuite,
			kAEDoObjectsExist,
			typeBoolean,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object in question",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 0 elements */
			},
			
			/* [9] */
			"get",
			"Get the data for an object",
			kAECoreSuite,
			kAEGetData,
			typeWildCard,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object whose data is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 1 elements */
				"as",
				keyAERequestedType,
				typeType,
				"the desired type for the data",
				optional,
				listOfItems,
				notEnumerated,
				reserved,
				reserved, reserved, reserved, reserved, reserved, reserved,
				reserved, reserved, reserved, reserved, reserved, reserved
			},
			/* [10] */
			"data size",
			"Return the size in bytes of an object",
			kAECoreSuite,
			kAEGetDataSize,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object whose data size is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 1 elements */
			},
			/* [11] */
			"move",		/*** NOTE: the insertion loc param is made optional. ***/
			"Move an object",
			kAECoreSuite,
			kAEMove,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to move",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 element */
				"to",
				keyAEInsertHere,
				typeInsertionLoc,
				"the new location for the object",
				optional,	/* default behavior is to move to front. */
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [12] */
			"set",
			"Set an object's data",
			kAECoreSuite,
			kAESetData,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			typeObjectSpecifier,
			"the object to change",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved, reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved, reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"to",
				keyAEData,
				typeWildCard,
				"the new value",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
			},
		},
		{	/* array Classes: 7 elements */
			/* [1] */
			"application",
			cApplication,
			"An application program",
			{	/* array Properties: 9 elements */
				/* [1] */
				"best type",
				pBestType,
				typeType,
				"the best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [2] */
				"class",
				pClass,
				typeType,
				"the class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [3] */
				"default type",
				pDefaultType,
				typeType,
				"the default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [4] */
				"frontmost",
				pIsFrontProcess,
				cBoolean,
				"Is this the frontmost application?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [5] */
				"name",
				pName,
				cIntlText,
				"the name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [7] */
				"version",
				pVersion,
				cVersion,
				"the version number of the application",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
			},
			{	/* array Elements: 1 element */
				cWindow,
				{	/* array KeyForms: 2 elements */
					formAbsolutePosition,
					formName
				},
			},
			/* [2] */
			"window",
			cWindow,
			"A window",
			{	/* array Properties: 15 elements */
				/* [1] */
				"bounds",
				pBounds,
				cQDRectangle,
				"the boundary rectangle for the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [2] */
				"position",
				pPosition,
				cQDPoint,
				"the upper left-hand coordinates of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"index",
				pIndex,
				cLongInteger,
				"the number of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [4] */
				"name",
				pName,
				cIntlText,
				"the title of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [5] */
				"best type",
				pBestType,
				typeType,
				"the best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [6] */
				"class",
				pClass,
				typeType,
				"the class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [7] */
				"default type",
				pDefaultType,
				typeType,
				"the default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [8] */
				"closeable",
				pHasCloseBox,
				cBoolean,
				"Does the window have a close box?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [9] */
				"titled",
				pHasTitleBar,
				cBoolean,
				"Does the window have a title bar?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [10] */
				"floating",
				pIsFloating,
				cBoolean,
				"Does the window float?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [11] */
				"modal",
				pIsModal,
				cBoolean,
				"Is the window modal?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [12] */
				"resizable",
				pIsResizable,
				cBoolean,
				"Is the window resizable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [13] */
				"zoomable",
				pIsZoomable,
				cBoolean,
				"Is the window zoomable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [14] */
				"zoomed",
				pIsZoomed,
				cBoolean,
				"Is the window zoomed?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
				/* [15] */
				"visible",
				pVisible,
				cBoolean,
				"Is the window visible?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved, reserved,
				notFeminine,
				notMasculine,
				singlular,
			},
			{	/* array Elements: 0 elements */
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 0 elements */
		},
	},
};

